"use strict";

require("core-js/modules/es.promise.js");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createRoot = createRoot;

function _path() {
  const data = require("@teambit/path.to-windows-compatible-path");

  _path = function () {
    return data;
  };

  return data;
}

function _lodash() {
  const data = require("lodash");

  _lodash = function () {
    return data;
  };

  return data;
}

function _path2() {
  const data = require("path");

  _path2 = function () {
    return data;
  };

  return data;
}

function _ui() {
  const data = require("./ui.aspect");

  _ui = function () {
    return data;
  };

  return data;
}

async function createRoot(aspectDefs, rootExtensionName, rootAspect = _ui().UIAspect.id, runtime = 'ui') {
  const rootId = rootExtensionName ? `'${rootExtensionName}'` : '';
  const defs = aspectDefs.filter(def => def.runtimePath);
  return `
import { Harmony } from '@teambit/harmony';
${getImportStatements(aspectDefs.map(def => def.aspectPath), 'Aspect')} 
${getImportStatements( // @ts-ignore no nulls can be found here - see above.
  defs.map(def => def.runtimePath), 'Runtime')}

Harmony.load([${getIdentifiers(aspectDefs.map(def => def.aspectPath), 'Aspect')}], '${runtime}', {})
  .then((harmony) => {
    harmony
      .run()
      .then(() => {
        const rootExtension = harmony.get('${rootAspect}');
        rootExtension.render(${rootId});
      })
      .catch((err) => {
        throw err;
      });
  });
  `;
}

function getImportStatements(extensionPaths, suffix) {
  return extensionPaths.map(path => `import ${getIdentifier(path, suffix)} from '${(0, _path().toWindowsCompatiblePath)(path)}';`).join('\n');
}

function getIdentifiers(extensionsPaths, suffix) {
  return extensionsPaths.map(path => `${getIdentifier(path, suffix)}`).join(', ');
}

function getIdentifier(path, suffix) {
  return (0, _lodash().camelCase)(`${(0, _path2().parse)(path).name.split('.')[0]}${suffix}`);
}

//# sourceMappingURL=create-root.js.map